package com.professor.zebrautility;

import android.app.Activity;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.ColorMatrix;
import android.graphics.ColorMatrixColorFilter;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.os.Looper;
import android.util.Base64;

import androidx.annotation.NonNull;

import com.zebra.sdk.comm.BluetoothConnection;
import com.zebra.sdk.comm.Connection;
import com.zebra.sdk.comm.ConnectionException;
import com.zebra.sdk.comm.TcpConnection;
import com.zebra.sdk.printer.PrinterLanguage;
import com.zebra.sdk.printer.ZebraPrinter;
import com.zebra.sdk.printer.ZebraPrinterFactory;
import com.zebra.sdk.printer.ZebraPrinterLanguageUnknownException;
import com.zebra.sdk.printer.discovery.BluetoothDiscoverer;
import com.zebra.sdk.printer.discovery.DiscoveredPrinter;
import com.zebra.sdk.printer.discovery.DiscoveryHandler;
import com.zebra.sdk.printer.discovery.NetworkDiscoverer;

import java.util.ArrayList;
import java.util.HashMap;

import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;

public class Printer implements MethodChannel.MethodCallHandler {

    private Connection printerConnection;
    private ZebraPrinter printer;
    private Context context;

    private MethodChannel methodChannel;

    private String selectedAddress = null;
    private String macAddress = null;
    private boolean tempIsPrinterConnect;
//    private FirebaseAnalytics mFirebaseAnalytics;

    private static ArrayList<DiscoveredPrinter> discoveredPrinters = new ArrayList<>();
    private static ArrayList<DiscoveredPrinter> sendedDiscoveredPrinters = new ArrayList<>();
    private static int countDiscovery = 0;
    private static int countEndScan = 0;


    public Printer(Context context, BinaryMessenger binaryMessenger) {
        this.context = context;
        this.methodChannel = new MethodChannel(binaryMessenger, "ZebraPrinterObject" + this.toString());
        methodChannel.setMethodCallHandler(this);

    }


    public static void discoveryPrinters(final Context context, final MethodChannel methodChannel) {
//        logEventFirebase("MyID", "Start discovery for printers");
        try {
            sendedDiscoveredPrinters.clear();
            for (DiscoveredPrinter dp :
                    discoveredPrinters) {
                addNewDiscoverPrinter(dp, context, methodChannel);
            }
            countEndScan = 0;
            BluetoothDiscoverer.findPrinters(context, new DiscoveryHandler() {
                @Override
                public void foundPrinter(final DiscoveredPrinter discoveredPrinter) {
                    discoveredPrinters.add(discoveredPrinter);
                    ((Activity) context).runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                            addNewDiscoverPrinter(discoveredPrinter, context, methodChannel);
                        }
                    });
                }

                @Override
                public void discoveryFinished() {
                    countEndScan++;
                    finishScanPrinter(context, methodChannel);
                }

                @Override
                public void discoveryError(String s) {
                    countEndScan++;
                    finishScanPrinter(context, methodChannel);
                }
            });

            NetworkDiscoverer.findPrinters(new DiscoveryHandler() {
                @Override
                public void foundPrinter(DiscoveredPrinter discoveredPrinter) {
                    addNewDiscoverPrinter(discoveredPrinter, context, methodChannel);

                }

                @Override
                public void discoveryFinished() {
                    countEndScan++;
                    finishScanPrinter(context, methodChannel);
                }

                @Override
                public void discoveryError(String s) {
                    countEndScan++;
                    finishScanPrinter(context, methodChannel);
                }
            });
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    private static void addPrinterToDiscoveryPrinterList(DiscoveredPrinter discoveredPrinter) {
        for (DiscoveredPrinter dp :
                discoveredPrinters) {
            if (dp.address.equals(discoveredPrinter.address))
                return;
        }

        discoveredPrinters.add(discoveredPrinter);
    }


    private void logEventFirebase(String id, final String name) {
//        try {
//            Bundle bundle = new Bundle();
//            bundle.putString(FirebaseAnalytics.Param.ITEM_ID, id);
//            bundle.putString(FirebaseAnalytics.Param.ITEM_NAME, name);
//            bundle.putString(FirebaseAnalytics.Param.CONTENT_TYPE, "text");
//            mFirebaseAnalytics.logEvent(FirebaseAnalytics.Event.SELECT_CONTENT, bundle);
//        } catch (Exception e){
//            System.out.println("Error in log event firebase: " + e.getMessage());
//        }

        ((Activity) context).runOnUiThread(new Runnable() {
            @Override
            public void run() {
                methodChannel.invokeMethod("logEvent"
                        , name);
            }
        });

    }

    private static void addNewDiscoverPrinter(final DiscoveredPrinter discoveredPrinter, Context context, final MethodChannel methodChannel) {

        addPrinterToDiscoveryPrinterList(discoveredPrinter);
        ((Activity) context).runOnUiThread(new Runnable() {
            @Override
            public void run() {
                for (DiscoveredPrinter dp :
                        sendedDiscoveredPrinters) {
                    if (dp.address.equals(discoveredPrinter.address))
                        return;
                }
                sendedDiscoveredPrinters.add(discoveredPrinter);
                if (discoveredPrinter.getDiscoveryDataMap().get("SYSTEM_NAME") != null) {
//                    logEventFirebase("MyID", "wifi printer found");
                    methodChannel.invokeMethod("printerFound"
                            , discoveredPrinter.getDiscoveryDataMap().get("SYSTEM_NAME")
                                    + ";" + discoveredPrinter.address);
                } else {
//                    logEventFirebase("MyID", "bluetooth printer found");
                    methodChannel.invokeMethod("printerFound"
                            , discoveredPrinter.getDiscoveryDataMap().get("FRIENDLY_NAME") + ""
                                    + ";" + convertMacAddressToMacAddressApp(discoveredPrinter.address));
                }
            }
        });
    }


    private static void finishScanPrinter(final Context context, final MethodChannel methodChannel) {
        if (countEndScan == 2) {
            if (discoveredPrinters.size() == 0) {
                if (discoveryPrintersAgain(context, methodChannel))
                    return;
            }
            ((Activity) context).runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    methodChannel.invokeMethod("onPrinterDiscoveryDone",
                            context.getResources().getString(R.string.done));
//                    logEventFirebase("MyID", "finish scanning with founded " + sendedDiscoveredPrinters.size());
                }
            });
        }
    }

    private static boolean discoveryPrintersAgain(Context context, MethodChannel methodChannel) {
        System.out.print("Discovery printers again");
        countDiscovery++;
        if (countDiscovery < 2) {
            discoveryPrinters(context, methodChannel);
            return true;
        }
        return false;
    }


    public void print(final String data) {
        new Thread(new Runnable() {
            public void run() {
//                enableTestButton(false);
                Looper.prepare();
                doConnectionTest(data);
                Looper.loop();
                Looper.myLooper().quit();
            }
        }).start();
    }


    private void doConnectionTest(String data) {

        if (printer != null) {
            printData(data);
        } else {
            disconnect();
        }
    }

    private void printData(String data) {
        try {
            byte[] configLabel = getConfigLabel(data);
            setStatus("Sending Data", Color.BLUE);
            printerConnection.write(configLabel);
            DemoSleeper.sleep(1500);

            if (printerConnection instanceof BluetoothConnection) {
//                String friendlyName = ((BluetoothConnection) printerConnection).getFriendlyName();
//                setStatus(friendlyName, Color.MAGENTA);
                DemoSleeper.sleep(500);
            }
            setStatus(context.getResources().getString(R.string.done), Color.BLUE);
        } catch (ConnectionException e) {
//            setStatus("Connection interrupt", Color.RED);
            disconnect();
        } finally {
            //disconnect();
        }
    }

    public boolean connectToSelectPrinter(String address) {
        selectedAddress = null;
        macAddress = null;
        logEventFirebase("MyID", "connect to printer: " + address);
        boolean isBluetoothPrinter;
        if (address.startsWith("B_")) {
            macAddress = address.split("_")[1];
            isBluetoothPrinter = true;
        } else {
            this.selectedAddress = address;
            isBluetoothPrinter = false;
        }
        printer = connect(isBluetoothPrinter);
        if (printer != null) return true;
        return false;
    }

    public String isPrinterConnect() {
        tempIsPrinterConnect = true;
        if (printerConnection != null && printerConnection.isConnected()) {
            new Thread(new Runnable() {
                public void run() {
//                enableTestButton(false);
                    try {
                        printerConnection.write("Test".getBytes());

                        if (printerConnection instanceof TcpConnection) {
                            TcpConnection printerConnectionTemp = new TcpConnection(getTcpAddress(), getTcpPortNumber());
                            try {
                                printerConnectionTemp.open();

                            } catch (ConnectionException e) {
                                disconnect();
                                tempIsPrinterConnect = false;

                            } finally {
                                printerConnectionTemp.close();
                            }
                        }
                    } catch (ConnectionException e) {
                        e.printStackTrace();
                        disconnect();
                        tempIsPrinterConnect = false;
                    }
                }
            }).start();
            if (tempIsPrinterConnect)
                return context.getString(R.string.connected);
            else return context.getString(R.string.disconnect);
        } else return context.getString(R.string.disconnect);
    }


    public ZebraPrinter connect(boolean isBluetoothPrinter) {

        String address;
        if (isPrinterConnect().equals(context.getString(R.string.connected)))
            disconnect();

        printerConnection = null;
        if (isBluetoothPrinter) {
            printerConnection = new BluetoothConnection(getMacAddress());
            address = getMacAddress();
        } else {
            try {

                printerConnection = new TcpConnection(getTcpAddress(), getTcpPortNumber());
                address = getTcpAddress();
            } catch (NumberFormatException e) {
                setStatus("Port Invalid", Color.RED);
                return null;
            }
        }
        try {
            printerConnection.open();

        } catch (ConnectionException e) {
//            setStatus("Comm Error! Disconnecting", Color.RED);
            DemoSleeper.sleep(1000);
            disconnect();
            logEventFirebase("MyID", "Error to connecting(1): " + e.getMessage() + "_" + address);
            return null;
        }

        ZebraPrinter printer = null;

        if (printerConnection.isConnected()) {
            try {
                printer = ZebraPrinterFactory.getInstance(printerConnection);
//                setStatus("Determining Printer Language", Color.YELLOW);
//                PrinterLanguage pl = printer.getPrinterControlLanguage();
//                setStatus("Printer Language " + pl, Color.BLUE);
            } catch (ConnectionException e) {
                //  setStatus("Unknown Printer Language", Color.RED);
                printer = null;
                DemoSleeper.sleep(1000);
                disconnect();
                logEventFirebase("MyID", "Error to connecting(2): " + e.getMessage() + "_" + address);
            } catch (ZebraPrinterLanguageUnknownException e) {
                // setStatus("Unknown Printer Language", Color.RED);
                printer = null;
                DemoSleeper.sleep(1000);
                disconnect();
                logEventFirebase("MyID", "Error to connecting(3): " + e.getMessage() + "_" + address);
            }
        }
        setStatus(context.getString(R.string.connected), Color.GREEN);
        logEventFirebase("MyID", "Printer s connected successfully" + "_" + address);
        return printer;
    }

    private String getMacAddress() {
        return macAddress;
    }

    private static String convertMacAddressToMacAddressApp(String macAddress) {
        return "B_" + macAddress;
    }

    private String getTcpAddress() {
//        return "192.168.60.163";
        return selectedAddress;
    }


    public void disconnect() {
        try {
            setStatus("Disconnecting", Color.RED);
            if (printerConnection != null) {
                printerConnection.close();
            }

        } catch (ConnectionException e) {
//            setStatus("COMM Error! Disconnected", Color.RED);
        } finally {
//            enableTestButton(true);
            setStatus(context.getString(R.string.disconnect), Color.RED);
        }
    }

    private void setStatus(final String message, int color) {
        ((Activity) context).runOnUiThread(new Runnable() {
            @Override
            public void run() {
                System.out.println("Printer set status: " + message);
                HashMap<String, Object> arguments = new HashMap<>();
                arguments.put("Address", getTcpAddress() != null
                        ? getTcpAddress() : convertMacAddressToMacAddressApp(getMacAddress()));
                arguments.put("Status", message);
                methodChannel.invokeMethod("changePrinterStatus", arguments);
            }
        });

    }

    private int getTcpPortNumber() {
//        return 9100;
        return 6101;
    }

    private byte[] getConfigLabel(String boardingPassData) {
        PrinterLanguage printerLanguage = printer.getPrinterControlLanguage();


        byte[] configLabel = null;
//        Bitmap bitmap = ((BitmapDrawable) context.getDrawable(R.drawable.boarding)).getBitmap();
//        ByteArrayOutputStream stream = new ByteArrayOutputStream();
//        bitmap.compress(Bitmap.CompressFormat.JPEG, 100, stream);
//        byte[] bitmapdata = stream.toByteArray();
        if (printerLanguage == PrinterLanguage.ZPL) {
            configLabel = "^XA^FO17,16^GB379,371,8^FS^FT65,255^A0N,135,134^FDTEST^FS^XZ".getBytes();
        } else if (printerLanguage == PrinterLanguage.CPCL) {
            String cpclConfigLabel = "! 0 200 200 406 1\r\n" + "ON-FEED IGNORE\r\n" + "BOX 20 20 380 380 8\r\n" + "T 0 6 137 177 TEST\r\n" + "PRINT\r\n";
            configLabel = cpclConfigLabel.getBytes();
        }


        configLabel = boardingPassData.getBytes();


        return configLabel;
    }

    public static String getZplCode(Bitmap bitmap, Boolean addHeaderFooter, int rotation) {
        ZPLConverter zp = new ZPLConverter();
        zp.setCompressHex(true);
        zp.setBlacknessLimitPercentage(50);
        Bitmap grayBitmap = toGrayScale(bitmap, rotation);
        return zp.convertFromImage(grayBitmap, addHeaderFooter);
    }

    public static Bitmap toGrayScale(Bitmap bmpOriginal, int rotation) {
        int width, height;
        bmpOriginal = rotateBitmap(bmpOriginal, rotation);
        height = bmpOriginal.getHeight();
        width = bmpOriginal.getWidth();

        Bitmap grayScale = Bitmap.createBitmap(width, height, Bitmap.Config.ARGB_8888);
        grayScale.eraseColor(Color.WHITE);
        Canvas c = new Canvas(grayScale);
        Paint paint = new Paint();
        ColorMatrix cm = new ColorMatrix();
        cm.setSaturation(0);
        ColorMatrixColorFilter f = new ColorMatrixColorFilter(cm);
        paint.setColorFilter(f);
        c.drawBitmap(bmpOriginal, 0, 0, paint);
        return grayScale;
    }


    public static Bitmap rotateBitmap(Bitmap source, float angle) {
        Matrix matrix = new Matrix();
        matrix.postRotate(angle);
        return Bitmap.createBitmap(source, 0, 0, source.getWidth(), source.getHeight(), matrix, true);
    }

    public void setSettings(String settings) {
        print(settings);
    }

    public void setDarkness(int darkness) {
        String setting = "             ! U1 setvar \"print.tone\" \"" + darkness + "\"\n";
        setSettings(setting);
    }

    public void setMediaType(String mediaType) {
        String settings;
        if (mediaType.equals("Label")) {
            settings = "! U1 setvar \"media.type\" \"label\"\n" +
                    "             ! U1 setvar \"media.sense_mode\" \"gap\"\n" +
                    //    "             ! U1 setvar \"print.tone\" \"0\"\n" +
                    "              ~jc^xa^jus^xz";
        } else if (mediaType.equals("BlackMark")) {
            settings = "! U1 setvar \"media.type\" \"label\"\n" +
                    "             ! U1 setvar \"media.sense_mode\" \"bar\"\n" +
                    //    "             ! U1 setvar \"print.tone\" \"0\"\n" +
                    "              ~jc^xa^jus^xz";
        } else {
            settings = //"! U1 SPEED 4\n" +
                    "      ! U1 setvar \"print.tone\" \"0\"\n" +
                            "      ! U1 setvar \"media.type\" \"journal\"\n";
            //   "      ! U1 setvar \"media.sense_mode\" \"bar\"" +
            // " ~jc^xa^jus^xz";
        }
        setSettings(settings);
    }

    private void convertBase64ImageToZPLString(String data, int rotation, MethodChannel.Result result) {
        try {
            byte[] decodedString = Base64.decode(data, Base64.DEFAULT);
            Bitmap decodedByte = BitmapFactory.decodeByteArray(decodedString, 0, decodedString.length);
            result.success(Printer.getZplCode(decodedByte, false, rotation));
        } catch (Exception e) {
            result.error("-1", "Error", null);
        }
    }

    @Override
    public void onMethodCall(@NonNull final MethodCall call, @NonNull final MethodChannel.Result result) {
        if (call.method.equals("print")) {
            print(String.valueOf(call.argument("Data")));
        } else if (call.method.equals("convertBase64ImageToZPLString")) {
            convertBase64ImageToZPLString(String.valueOf(call.argument("data"))
                    , Integer.valueOf(call.argument("rotation").toString()), result);
        } else if (call.method.equals("disconnect")) {
            new Thread(new Runnable() {
                @Override
                public void run() {
                    disconnect();

                }
            }).start();
        } else if (call.method.equals("isPrinterConnected")) {
            result.success(isPrinterConnect());
        } else if (call.method.equals("discoverPrinters")) {
            discoveryPrinters(context, methodChannel);
            result.success(null);
        } else if (call.method.equals("setMediaType")) {
            String mediaType = call.argument("MediaType");
            setMediaType(mediaType);
        } else if (call.method.equals("setSetting")) {
            String settingCommand = call.argument("SettingCommand");
            setSettings(settingCommand);
        } else if (call.method.equals("setDarkness")) {
            int darkness = call.argument("Darkness");
            setDarkness(darkness);
        } else if (call.method.equals("connectPrinter")) {
            new Thread(new Runnable() {
                @Override
                public void run() {
                    final boolean r = false;
                    connectToSelectPrinter(String.valueOf(call.argument("address")));
                    ((Activity) context).runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                            if (r) result.success(true);
                            else result.success(false);
                        }
                    });
                }
            }).start();

        } else {
            result.notImplemented();
        }
    }
}